#  Perl snippet inserting dh_ada_library in the dh debhelper sequencer
#
#  Copyright (C) 2012-2020 Nicolas Boulenguez <nicolas@debian.org>
#
#  This program is free software: you can redistribute it and/or
#  modify it under the terms of the GNU General Public License as
#  published by the Free Software Foundation, either version 3 of the
#  License, or (at your option) any later version.
#  This program is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
#  General Public License for more details.
#  You should have received a copy of the GNU General Public License
#  along with this program. If not, see <http://www.gnu.org/licenses/>.
#
use warnings;
use strict;
use Debian::Debhelper::Dh_Lib;

# See WHEN section of /usr/share/doc/dh-ada-library/README.
insert_after("dh_lintian", "dh_ada_library");

1;
